#!/usr/bin/python3
# -*- coding: utf-8 -*-
'''
________________________________________________________________________
 a_datum.py                                                        
 This file is free software under         
     the terms of the GPL without any warranty - see the file COPYING 
 Diese Datei ist Freie Software gemäß
             der GPL ohne jegliche Garantie - siehe die Datei COPYING 
________________________________________________________________________

 a_datum.py stellt Tools für die anderen absenzen-Module zur Verfügung
  Schwerpunkt: Datum-Berechnungen und -Umwandlungen
________________________________________________________________________
'''
##Importe
import time, datetime

from a_konstanten import *
import a_db

def ist_isodatum(datum):
    if  len(datum)==10 and \
        datum[:2]=='20' and  \
        datum[4]=='-' and  \
        datum[7]=='-' and  \
        datum[2:4].isdigit() and \
        datum[5:7].isdigit() and \
        datum[8:10].isdigit() and \
        int(datum[5:7])<13 and \
        int(datum[8:10])<32:
            return True
    return False

def isodatum2datum(isodatum,incaseoferror='?nodate?'):
    if len(isodatum)!=10:     return incaseoferror
    t,m,j= isodatum[8:],isodatum[5:7],isodatum[:4]
    return t+"."+m+"."+j

def isodatum2tagmonat(isodatum,incaseoferror='?nodate?'):
    if len(isodatum)!=10:     return incaseoferror
    t,m= isodatum[8:],isodatum[5:7]
    return t+"."+m+"."

def datum2isodatum(datum,strict=False):
    if strict and ( len(datum)!=10 or datum[2]!="." or datum[5]!="."  ): return '?nodate?'
    if len(datum)!=10:    return '?nodate?'
    t,m,j= datum[:2],datum[3:5],datum[6:]
    if not t.isdigit() or not m.isdigit() or not j.isdigit(): return '?nodate?'
    return j+"-"+m+"-"+t

def isodatum2int(isodatum):
    t,m,j= isodatum[8:],isodatum[5:7],isodatum[:4]
    return int(t),int(m),int(j)

def wochentag(isodatum):
    ts=time.strptime(isodatum,'%Y-%m-%d')
    return ts.tm_wday
    
def wochentagXX(isodatum):
    return wotage[wochentag(isodatum)]
    
def isodatum_plus_tage(isodatum,tage):
    t,m,y=isodatum2int(isodatum)
    delta =datetime.timedelta(tage)
    mydate=datetime.date(y,m,t)+delta
    return mydate.isoformat()
    
def kalenderwoche(isodatum):
    t,m,y=isodatum2int(isodatum)
    mydate=datetime.date(y,m,t)
    return mydate.isocalendar()[1]
    
def kalendermonat(isodatum):
    t,m,y=isodatum2int(isodatum)
    mydate=datetime.date(y,m,t)
    return mydate.month
    
def reales_schuljahr():
    t,m,j=isodatum2int(_heutedatum_)
    return j if m>=9 else j-1

def ist_freier_tag(datum):
    ob,warum=ist_freier_tag_warum(datum)
    return ob
    
def ist_freier_tag_warum(datum):
    rowdict=a_db.get_one_rowdict('''
            SELECT count(name),name
              FROM noschool 
             WHERE (? BETWEEN erstertag AND letztertag)
                OR (? BETWEEN erstertag AND letztertag)''',(datum,'9999'+datum[4:]))
    if rowdict:
        if rowdict[0]>0:
            return True, rowdict[1]
    return False,''

def letzter_halbjahresbeginn():
    startmonat=9 if _timestamp_.tm_mon>8 or _timestamp_.tm_mon<3 else 1
    starttag=10 if startmonat==9 else 5
    if _timestamp_.tm_mon<9 and startmonat>8: # wenn jetzt 1. HJ, Anzeige soll 2. HJ zeigen: also früher im Schuljahr = letztes Kalenderjahr
        startjahr=_timestamp_.tm_year-1
    elif _timestamp_.tm_mon>8 and startmonat<9: # und umgekehrt
        startjahr=_timestamp_.tm_year+1
    else:
        startjahr=_timestamp_.tm_year
    return startjahr,startmonat,starttag

def heute_plus_delta_datum(delta):
    return timestamp2isodatum(ts_verschiebe(_timestamp_,delta))
    
def timestamp2isodatum(ts):
    return time.strftime('%Y-%m-%d',ts)

def ts_verschiebe(ts,delta_tage):
    return time.localtime(time.mktime((ts.tm_year,ts.tm_mon,ts.tm_mday+delta_tage,0,0,0,0,0,-1)))
    
def ts_erstellen(jahr,monat,tag):
    return time.localtime(time.mktime((jahr,monat,tag,0,0,0,0,0,-1)))

def get_now():          return _now_
def get_jetzt():        return _jetzt_
def get_timestamp():    return _timestamp_
def get_heutedatum():   return _heutedatum_
def get_startjahr():    return _startjahr_

_now_=time.time() #sec since epoche
_timestamp_=time.localtime(_now_)
_jetzt_=time.strftime('%d. %B. %Y %H:%M',_timestamp_)
_heutedatum_=timestamp2isodatum(_timestamp_)    # im Format 2014-04-21
_startjahr_=_timestamp_.tm_year # Jahr des Schuljahresbeginns
if _timestamp_.tm_mon<9: _startjahr_-=1

