#!/usr/bin/python3
# -*- coding: utf-8 -*-
'''
________________________________________________________________________
 a_termine.py 
 This file is free software under         
     the terms of the GPL without any warranty - see the file COPYING 
 Diese Datei ist  Freie Software gemäß
             der GPL ohne jegliche Garantie - siehe die Datei COPYING 
________________________________________________________________________

 a_termine.py verwaltet Temine. 
 Attribute eines Termins:
    tid,autor,name,kommentar,datum,datumende,zeit,
    typ:    0= von Lehrer für Klasse/Kurs gesetzt (Info)
            1= für Klasse gesetzte Schulaufgabe
            2= vom Terminverwalter für Klasse/Kurs gesetzter Termin (offiziell)

            3= für Q11 gesetzte Schulaufgabe
            4= für Q12 gesetzte Schulaufgabe
            5= Termin für Q11
            6= Termin für Q12

            7= Termin für Schulleitung (nur dieser zugänglich zu machen)
            8= Termin für Lehrer (nur Lehrern zugänglich zu machen)
            9= Allgemeiner Termin (kann veröffentlicht werden)
            
           10= Fremdtermin - ohne Betroffene oder Verantwortliche im Haus, z.B. religiöse Feiertage

    status:-1: gelöscht
            0: gültig
            5: Entwurf, History verwerfen
    betroffene,
    verantwortliche,history
________________________________________________________________________
'''

import cgi, cgitb
cgitb.enable()

from a_konstanten import *

import a_tool
import a_db
import a_datum
import a_js, a_h

import a_session as a_s

veroeffentlichbare_termintypen=[2,5,6,9] # für Homepage und Jahresbericht
komplexe_termintypen=[2,5,6,7,8,9] # mit Endedatum und Uhrzeit sowie Verantwortlichen

ttypnamen=['Termin für Klasse/Kurs','Schulaufg./Kurza. 5-10','Termin f. Klassen 5-10 (offiziell)','Schulaufg. Q11','Schulaufg. Q12','Termin für Q11','Termin für Q12','Termin für Schulleitung','Termin für Lehrer','Allgemeiner Termin','Fremdtermin']
stati_anzuzeigen_in={   "Kalender":        [0,1],
                        "Edit":         [-1,0,1],
                        "Verwalter":    [-1,0,1,5],
                        "Terminleiste": [-1,0,1],
                        "BlaBlubb":     [-1,0],
                    } # wird noch nirgends genutzt
terminhilfe='''<b>Termineingabe</b>
Ein Termin muss mindestens ein Datum und einen Namen (z.B. M3) haben.
Der Name sollte kurz sein, denn nur seine ersten 4 Zeichen werden in der Absenzentabelle direkt angezeigt.
Ist das "Datumende" ein späteres Datum, so wird der Termin als Zeitraum gespeichert.'''

g_t_verwaltung_hilfe='''<b>Verwaltung allgemeiner Termine</b>
Termine können nur angelegt, eingeschränkt geändert und als gelöscht markiert werden.<br>
'''

highlightdatum='none'
return_to_classicview=False

class A_Termin:
    def __init__(self,rowdict):
        self.tid        =rowdict.get("tid",0)
        self.autor      =rowdict.get("autor","a-unbekannt")
        self.name       =rowdict.get("name","")
        self.kommentar  =rowdict.get("kommentar","")
        self.datum      =rowdict.get("datum","2000-01-01")
        self.datumende  =rowdict.get("datumende",self.datum)
        self.zeit       =rowdict.get("zeit","")
        self.typ        =rowdict.get("typ",0)
        self.status     =rowdict.get("status",0)
        self.betroffene =rowdict.get("betroffene","")
        self.verantwortliche=rowdict.get("verantwortliche","")
        self.history    =rowdict.get("history","")
        self.tag_jb     =rowdict.get("tag_jb",1)
        self.tag_hp     =rowdict.get("tag_hp",1)
        self.modifiziert=rowdict.get("modifiziert",'2000-01-01')
    
    @classmethod
    def lade(cls, tid):
        '''liest einen Termin aus der db ein, legt ihn als Klasse an, und gibt ihn zurück!
        Bei Erfolg ist tid in der Klasse richtig gesetzt, sonst der Wert 0'''
        #rowdict=a_db.get_one_rowdict('''SELECT tid,autor,name,kommentar,datum,datumende,zeit,typ,status,betroffene,verantwortliche,history
        #                            FROM termine
        #                            WHERE tid=?''',(tid,))
        rowdict=a_db.get_one_rowdict('''SELECT *
                                    FROM termine
                                    WHERE tid=?''',(tid,))
        if rowdict:
            return cls(rowdict)
        return cls( {"tid":0} )
        
    @classmethod
    def lade_liste(cls,typlist,statuslist,vondatum,bisdatum,nur_j=False,nur_h=False):
        typauswahl=','.join([ str(s) for s in typlist if type(s) is int ])
        statusauswahl=','.join([ str(s) for s in statuslist if type(s) is int ])
        jh_filter =" AND tag_jb>0 " if nur_j else ""
        jh_filter+=" AND tag_hp>0 " if nur_h else ""
        endedatumswahl='OR datumende BETWEEN :von AND :bis' if nur_h else ''
        rowdict_liste=a_db.get_rowdict_list('''SELECT *
                            FROM termine 
                            WHERE typ    in ('''+typauswahl+   ''') 
                              AND status in ('''+statusauswahl+''') 
                              AND ( datum BETWEEN :von AND :bis '''+endedatumswahl+''' )
                              '''+jh_filter+'''
                            ORDER BY datum ASC, status DESC''', {"von": vondatum,"bis": bisdatum})
        liste=[]
        for rowdict in rowdict_liste:
            liste.append(cls(rowdict))
        return liste

    @classmethod
    def lade_tages_liste(cls,typlist,datum,auchzeitraum=False):
        # Zuerst eine in die SQL-Abfrage einzubindende Menge gesuchter Typen erzeugen
        typauswahl=','.join([ str(s) for s in typlist if type(s) is int ])
        if auchzeitraum:
            rowdict_liste=a_db.get_rowdict_list('''SELECT *
                            FROM termine 
                            WHERE ? BETWEEN datum AND datumende
                              AND typ in ('''+typauswahl+''') 
                            ORDER BY status DESC, betroffene ASC''',(datum,))
        else:
            rowdict_liste=a_db.get_rowdict_list('''SELECT *
                            FROM termine 
                            WHERE datum = ?
                              AND typ in ('''+typauswahl+''') 
                            ORDER BY status DESC, betroffene ASC''',(datum,))
        liste=[]
        for rowdict in rowdict_liste:
            liste.append(cls(rowdict))
        return liste
        
    def als_titel(self):
        return self.name+' ('+a_tool.lesbare_betroffene(self.betroffene)+'): '+self.kommentar
            
    def als_popup(self):
        geloescht=True if self.status<0 else False
        popuptxt='<b>'
        if geloescht: popuptxt+='<schwarz>'
        popuptxt+=self.name+'&nbsp;'
        if self.status==5:
            popuptext+='Entwurf! '
        elif self.status==1:
            popuptext+='(JB) '
        if   self.typ in [1,3,4] and not geloescht: popuptxt+='<rosa>Schulaufgabe in '
        elif self.typ==1 :                  popuptxt+='Schulaufgabe in '
        else:                               popuptxt+='für '
        popuptxt+=a_tool.lesbare_betroffene(self.betroffene) if self.typ!=9 else ' <i>alle</i>'
        if   self.typ in [1,3,4] and not geloescht: popuptxt+='</rosa>'
        popuptxt+='&nbsp;'*6+' ('+self.autor+')'
        if geloescht: popuptxt+='</schwarz>'
        popuptxt+='</b>'
        if self.kommentar:
            popuptxt+=a_tool.lf2br(self.kommentar)+'<br>'
        popuptxt+=self.history.split('\n')[-1]+'<br>'
        return popuptxt

    def als_kalenderzeile(self,showonly=True,toedittid=0,user_ist_terminverwalter=False,keineverantwortlichen=False,merge3=False):
        w1=wotage[a_datum.wochentag(self.datum)]     
        w2=wotage[a_datum.wochentag(self.datumende)] if a_datum.ist_isodatum(self.datumende) else ''
        dl=6 if showonly else 10 # datum limitierer
        if self.datum != self.datumende and w2:
            if not merge3:
                datumzelle=w1+'&nbsp;'+a_datum.isodatum2datum(self.datum)[:dl]+'<br>-&nbsp;'+w2+'&nbsp;'+a_datum.isodatum2datum(self.datumende)[:dl]
            else:
                datumzelle='&nbsp;'+a_datum.isodatum2datum(self.datum)[:dl]+'&nbsp;-&nbsp;'+a_datum.isodatum2datum(self.datumende)[:dl]
        else:
            datumzelle=w1+'&nbsp;'+a_datum.isodatum2datum(self.datum)[:dl]
        tdclass=' geloescht' if self.status<0 else ( ' entwurf' if self.status==5 else "" )
        if self.typ in [10]:
            tdclass+=' fremd'
        if self.tid == toedittid:
            tdclass=' blau'
        if self.typ in [7,8]:
            tdclasstyp=' rot'
        elif self.typ in [2]:
            tdclasstyp=' darkorange'
        elif self.typ in [10]:
            tdclasstyp=' fremd'
        else:
            tdclasstyp=tdclass
        td='<td class="'+tdclass+'">{}</td>'
        if self.typ in [10]:
            tdb='<td class="'+tdclass+'">{}</td>'
        else:
            tdb='<td class="'+tdclass+'"><b>{}</b></td>'
        tdtyp='<td class="'+tdclasstyp+'">{}</td>'
        tdaktion='<td class="'+tdclass+'" style="text-align:right;line-height: 90%;">{}</td>'
        diebetroffenen=TERMIN_BETROFFENE[self.typ] if self.typ!=2 else a_tool.lesbare_betroffene(self.betroffene)
        merge_betroffene= '' if self.typ!=2 else a_tool.lesbare_betroffene(self.betroffene)+' '
        zeileskelett = td + ((td + tdb)  if not merge3 else '')  + tdtyp + (td  if not merge3 else '') + (td  if not keineverantwortlichen and not merge3 else '') + ( tdaktion if not showonly else '')
        if showonly:
            if merge3: # FIXME: dämliche Fallunterscheidung wegen unterschiedlicher Parameterzahl - geht doch intelligenter, oder?
                merger=merge_betroffene+a_tool.lf2br(self.kommentar)+ (' durch '+self.verantwortliche if len(self.verantwortliche.strip())>0 else '')
                fertigezeile=zeileskelett.format(datumzelle.split(';',maxsplit=1)[1],self.name+merger)
            else:
                if keineverantwortlichen:
                    fertigezeile=zeileskelett.format(datumzelle,self.zeit,self.name,
                                                    diebetroffenen,
                                                    a_tool.lf2br(self.kommentar))
                else:
                    vera=a_tool.entferne_titel(self.verantwortliche) if showonly else self.verantwortliche 
                    fertigezeile=zeileskelett.format(datumzelle,self.zeit,self.name,
                                                    diebetroffenen,
                                                    a_tool.lf2br(self.kommentar),vera)
            return fertigezeile
        if user_ist_terminverwalter or self.typ in (5,6):
            if self.status<0:
                link=' gelöscht \n'
            else:
                link='<a href="a_termine.py?tid2delete='+str(self.tid)+'&tdelsid='+str(a_s.aS.sid)+'">Löschen</a>'
            edit='<a href="a_termine.py?toedittid='+str(self.tid)+'">Bearb.</a>&nbsp;<a href="a_termine.py?kopietid='+str(self.tid)+'">Kopie</a><br>\n'
        else:
            edit=''
            link=''
        buttons_n_history='' if showonly else (link+edit+'<small>'+a_tool.lf2br(self.history,-3))
        tagliste='(jb)' if self.tag_jb else ''
        tagliste+=' (hp)' if self.tag_hp else ''
        tagliste+=(('<br>' if tagliste else '') + '<b><i>ENTWURF</i></b>') if self.status==5 else ''
        
        if tagliste:
            tagliste='<br><span style="color:magenta;">'+tagliste.strip()+'</span>'
        return  zeileskelett.format(datumzelle,self.zeit,self.name,
            (TERMIN_BETROFFENE[self.typ] if self.typ!=2 else a_tool.lesbare_betroffene(self.betroffene))+tagliste,
            a_tool.lf2br(self.kommentar),self.verantwortliche,buttons_n_history)
    
    def als_tageszeile(self,showonly=True,toedittid=0):
        tdclass=' geloescht' if self.status<0 else ""
        if self.tid == toedittid:
            tdclass=' blau'
        if self.typ in [1,3,4]:
            tdclass2=' rot'
            sa_spalte='Schul-<br>aufgabe'
        else:
            tdclass2=tdclass
            sa_spalte='Termin'
        if self.status<0:
            link=' gelöscht \n'
        else:
            link='<a href="a_termine.py?tid2delete='+str(self.tid)+'&tdelsid='+str(a_s.aS.sid)+'">Löschen</a> '
        betsy=[]
        for klasse in self.betroffene.split():
            if klasse.startswith('@'):
                betsy.append('<i>'+klasse[1:]+'</i>')
            else:
                betsy.append(klasse)
        bet_spalte=', '.join(betsy)
        td='<td class="'+tdclass+'">{}</td>'
        td2='<td class="'+tdclass2+'">{}</td>'
        td3='<td class="'+tdclass+'" style="text-align:right;">{}</td>'
        edit='<a href="a_termine.py?toedittid='+str(self.tid)+'">Bearb.</a>\n'
        zeileskelett = td + td2*2 + td*2 + ( td3 if not showonly else '')
        buttons_n_history='' if showonly else (link+edit+'<br><small>'+a_tool.lf2br(self.history,-3))
        return  zeileskelett.format(self.name,sa_spalte,bet_spalte,
                a_tool.lf2br(self.kommentar),self.autor,buttons_n_history)
        

    @staticmethod
    def get_tagestitel(showonly):
        return '''<th>Name</th><th>SA?</th><th>Gilt für</th><th>Beschreibung</th><th>Autor</th>''' + \
                ('<th>Aktionen</th>' if not showonly else '')
    
    @staticmethod
    def get_kalendertitel(showonly,keineverantwortlichen=False,merge3=False):
        return '<th>Datum</th>'+ \
                ('<th>Zeit</th>' if not merge3 else '') + \
                ('<th>Titel</th>'  if not merge3 else '')+ \
                ('<th>Gilt für</th>' if not merge3 else '') + \
                '<th>Beschreibung</th>' +\
                ('<th>Verantwortliche</th>' if not (keineverantwortlichen or merge3) else '') + \
                ('<th>Aktionen</th>' if not showonly else '')
    
    @staticmethod
    def get_kalenderdummy(showonly):
        leerspalten="6" if showonly else "7"
        return '<td colspan="'+leerspalten+'''" style="text-align:center;" class="geloescht">
                <small><small>kein Termin in dieser Kalenderwoche</small></small></td></tr>\n'''

        
def termine_dialog(editdatum="",toedittid=0,resultat="",noedit=False,kopieanlegen=False,showclassicview=False,zielklasse=''):
    # HTML Basis
    a_tool.prt(a_h.einleitung()+a_h.headstart()+'''
    <meta http-equiv="refresh" content="{timeout}; URL=a_main.py" />\n'''.format(timeout=a_s.aS.get_loginzeit()+10))
    a_tool.prt('<script src="../js/jquery.js"></script>\n')
    if not showclassicview:
        a_tool.prt('<script src="../js/a_teingabe.js"></script>\n')
    else:
        a_tool.prt('<script src="../js/a_scroll2highlight.js"></script>\n')
    a_tool.prt('</head>\n<body style="padding-bottom:10px;text-align:center;"><h1>{}</h1>'.format('Schulaufgabenplan' if showclassicview else 'Terminverwaltung'))
    logo=a_tool.get_logo()
    a_tool.prt('<div class="b1" style="text-align:center">'+logo+'</div>\n')
    
    # HTML Links
    a_tool.prt('<div class="s2 gradientbuttons">')
    a_tool.prt('''<div style="float:left;"><a href="a_main.py">Zur Absenzenverwaltung</a></div>\n''')
    if not showclassicview:
        a_tool.prt('''<div class="gtermine"  style="float:left;">
            <a href="a_termine.py?showclassicview=1">Zum Schulaufgabenplan</a>
            </div>\n''')
    if a_s.aS.ist_terminverwalter():
        listtypen=list(range(10))    
    elif a_s.aS.ist_kurs_verwalter():
        listtypen=[0,1,3,4,5,6]
    else:
        listtypen=[0,1]
    if a_s.aS.ist_terminverwalter() or a_s.aS.ist_kurs_verwalter():
        a_tool.prt('''<div class="gtermine"  style="float:left;">
        <a href="a_termine.py?verwalten=1">Zur Terminverwaltung</a>
        </div>\n''')
        a_tool.prt('''<div class="gtermine"  style="float:left;">
        <a href="a_termine.py?grab=qj" target="_blank">Liste für Jahresbericht</a>
        <a href="a_termine.py?grab=qh" target="_blank">Liste für Homepage</a>
        <a href="a_termine.py?grab=ql" target="_blank">Liste für Lehrer</a>
        </div>\n''')
    else:
        a_tool.prt('''<div class="gtermine"  style="float:left;">
            <a href="a_termine.py">Zur Terminliste</a>
            </div>\n''')
    a_tool.prt('</div>\n')
    
    # Ausgabe von Ergebnissen vorheriger Aktionen
    if resultat:
        if   resultat[0]=="-":  style='background-color:#FC6;'
        elif resultat[0]=="+":  style='background-color:lightgreen;'
        elif resultat[0]=="!":  style='color:blue;'
        elif resultat[0]=="#":  style='color:darkgreen;'
        else: style=""
        a_tool.prt('<div class="s2" style="text-align:center;'+style+'">'+resultat[1:]+'</div>\n')
    
    # Verlassen, wenn Schulaufgabenplan gewünscht
    if showclassicview:
        a_tool.prt(schulaufgabenplan(showonly=(a_s.aS.kuerzel=='Gast')))
        a_tool.prt('</body></html>')
        return

    ###### Eigentliches Editieren beginnt ##########
    if editdatum or toedittid:
        #################
        ## Termineinlesen oder Dummywerte zum Editieren

        if toedittid:
            dialog_termin=A_Termin.lade(toedittid)
            if dialog_termin.tid and not ( dialog_termin.typ>6 and not a_s.aS.ist_terminverwalter() ):
                edittop=a_tool.headhelper('# ⟳',zeichenfarbe='#339',titel='Termin bearbeiten',hilfetext=terminhilfe)
                editdatum=dialog_termin.datum
            else:
                toedittid=0
                editdatum=a_datum.get_heutedatum()
        if not toedittid:
            edittop=a_tool.headhelper('# +',zeichenfarbe='#339',titel='Neuer Termin',hilfetext=terminhilfe)
            dialog_termin=A_Termin( {"tid":0,"kuerzel":a_s.aS.kuerzel,"datum":editdatum } )
            if a_tool.getcgiinteger('sa')==1:
                dialog_termin.typ=1
        
    if a_s.aS.ist_terminverwaltung() and  ( ( not toedittid and not editdatum) or noedit):
        a_tool.prt( grosse_terminverwaltung(toedittid) )

    elif (editdatum or toedittid):
        a_tool.prt( termin_tageslisten_div(editdatum,toedittid,showonly,listtypen) )
        
    if not noedit and (editdatum or toedittid):
        if kopieanlegen:
            dialog_termin.tid=0
            edittop=a_tool.headhelper('# ⇗',zeichenfarbe='#339',titel='Kopie des Termins anlegen ',hilfetext=terminhilfe)
        if a_s.aS.ist_terminverwaltung() and dialog_termin.tid==0:
            if a_s.aS.ist_kurs_verwalter():     dialog_termin.typ=5
            else:                               dialog_termin.typ=9
        if zielklasse=='@Q11':   
            zielklasse=''
            dialog_termin.typ=3
            if not dialog_termin.name:
                dialog_termin.name='Q11SA'
        elif zielklasse=='@Q12': 
            zielklasse=''
            dialog_termin.typ=4
            if not dialog_termin.name:
                dialog_termin.name='Q12SA'

        termin_eingabebox(dialog_termin,edittop,kopieanlegen=kopieanlegen,zielklasse=zielklasse)
        
    a_tool.prt('</body></html>')
    
def termin_tageslisten_div(datum,toedittid,showonly=True,typen=[0,1]):
    tali=[]
    tali.append('<div class="s1  "  style="background-color:#E3E3E3;" >') #gradientbuttons
    tali.append(a_tool.headhelper('# ?',zeichenfarbe='#339', \
        titel='Auswahl der Termine am '+a_datum.isodatum2datum(datum)+' &nbsp; <span class="gradientbuttons"><a href="a_termine.py?editdatum='+datum+'">Neuer&nbsp;Termin</a></span>', \
        hilfetext=terminhilfe)+'<br>')
    termin_obj_liste=A_Termin.lade_tages_liste(typen,datum)
    zaehler=0
    tali.append('<table class="terminplaner"><tr>'+A_Termin.get_tagestitel(showonly)+'</tr>')
    for loop_termin in termin_obj_liste:
        zaehler+=1
        tr='<tr class="ungerade">' if zaehler%2 else '<tr class="gerade">' 
        tali.append(tr+loop_termin.als_tageszeile(showonly,toedittid)+'</tr>')
    tali.append('</table></div>')
    return "\n".join(tali)
    
def t_e_b_auswahlhandler(nr,typ):
    flag= 'checked="checked"'  if typ==nr else ''
    return a_tool.text_lur('&nbsp;'*5+'<input type="radio" name="ttyp" id="ttyp{nr}" value="{nr}" {flag}><label for="ttyp{nr}">{name}</label><br>'.format(nr=nr,flag=flag,name=ttypnamen[nr]),'')
    
def termin_eingabebox(dialog_termin,edittop,kopieanlegen=False,zielklasse=''):
    #################
    ## Termineingabe
    # Ziel:
    # Oberstufe? Dann darf ein Verwalter jeden Kurs wählen und darf Schulaufgaben eintragen (3-6)
    #            Jeder darf für seinen Kurs Termine ohne Schulaufgabe eintragen (0)
    # UM? Jeder darf in seiner Jahrgangsstufe (nicht nur Klasse) Schulaufgaben und Infos eintragen (0,1)
    # Terminverwalter? Darf Termine des Typs 2,5&6,7,8,9 eintragen
    teb=['<div class="s2 w45"  style="background-color:#E3E3E3;" >\n'+edittop]
    nurklassen=not a_s.aS.ist_terminverwaltung() 
    nurQSA=dialog_termin.typ in [3,4] and a_s.aS.ist_kurs_verwalter() # nur Schulaufgaben für Q-Stufe
    kurs=a_s.aS.get_kurs()
    js=a_s.aS.get_aktjs()
    selektor=[]
    kk_auswahl=False
    kurszahl=0
    klassenzahl=0
    if not zielklasse.startswith("@"): # Garant für eine UM-Klasse - dann keine Kurse anzeigen
        selektor=['<select id="tkurse" name="tkurse" size="8" multiple>']
        if a_s.aS.ist_kurs_verwalter():
            for row in a_db.get_rowdict_list('SELECT DISTINCT kbez FROM p2k ORDER BY kbez'): # nur Kurse, die auch belegt sind...:
                ausgewaehlt=' selected' if row["kbez"] in dialog_termin.betroffene else ''
                selektor.append('\t<option'+ausgewaehlt+'>'+row[0]+'</option>')
                kurszahl+=1
        else:
            if kurs[0]!='@' and a_s.aS.get_aktjs()>=21:
                selektor.append('\t<option selected>'+kurs+'</option>')
        selektor.append('</select>')

    if not nurQSA:
        selektor.append('<select name="tklassen" size="8" multiple>')
        if nurklassen:
            rowdict_list=a_db.get_rowdict_list('''SELECT DISTINCT klassen.name
                                FROM klassen
                                WHERE klassen.js in 
                                        (SELECT DISTINCT klassen.js 
                                         FROM klassen,lehrerinklasse 
                                         WHERE (kname=name AND user_nr= :nr )
                                            OR klassleiter_nr= :nr
                                        )
                                ORDER BY  klassen.js, klassen.name''', \
                        {"nr":a_s.aS.user_nr,}) # nur Klassen, deren jahrgangsstufe der Lehrer unterrichtet
        else:
            rowdict_list=a_db.get_rowdict_list('''SELECT DISTINCT klassen.name FROM klassen''')
            
        for row in rowdict_list:
            ausgewaehlt=' selected' if '@'+row[0] in dialog_termin.betroffene+' '+zielklasse else ''
            selektor.append('\t<option'+ausgewaehlt+'>'+row[0]+'</option>')
            klassenzahl+=1
        selektor.append('</select>')
    tkk_auswahl="\n".join(selektor)
    selektor=None
    
    teb.append('<form action="a_termine.py" method="post" ><div style="clear:left;float:left;width:50%;">')
    teb.append(a_tool.text_lur('Autor:',a_s.aS.klarname))
    if kopieanlegen:
        teb.append(a_tool.text_lur('Datum:','<input name="tdatum" value="'+a_datum.isodatum2datum(dialog_termin.datum)+'" type="text" size="10" maxlength="12" />' ))
    else:
        teb.append(a_tool.text_lur('Datum:',a_datum.isodatum2datum(dialog_termin.datum))+ '<br>\n')
            #'<input name="tdatum" value="'+a_datum.isodatum2datum(dialog_termin.datum)+'" type="hidden" /><br>\n')
    if ((dialog_termin.name or toedittid) and not a_s.aS.ist_terminverwalter) or nurQSA:
        teb.append(a_tool.text_lur('Name:','',dialog_termin.name))
        teb.append('<input name="tname" value="'+dialog_termin.name+'" type="hidden" /><br>\n')
    else:
        teb.append(a_tool.text_lur('Name (z.B. "M3"):','<input name="tname" value="'  +dialog_termin.name +'" type="text" size="15" maxlength="'+str(TERMIN_NAMENLAENGE)+'" />' ))
    if a_s.aS.ist_terminverwalter() and \
        ( (not dialog_termin.tid and not nurklassen) or \
          dialog_termin.typ in komplexe_termintypen ):
            teb.append(a_tool.text_lur('DatumEnde:','<input name="tdatumende" value="'  +a_datum.isodatum2datum(dialog_termin.datumende) +'" type="text" size="10" maxlength="10" />' ))
            teb.append(a_tool.text_lur('Zeit (z.B. 3. Std.):','<input name="tzeit" value="' +dialog_termin.zeit +'" type="text" size="10" maxlength="'+str(TERMIN_ZEITLAENGE)+'" />' ))
    
    teb.append(a_tool.text_lur('Termintyp:<br>',''))
    hat_ab=False
    hat_o=False
    hat_c=False
    hat_def=False
    if not nurQSA and not a_s.aS.ist_terminverwaltung() and (not dialog_termin.tid or dialog_termin.typ in [0,1]):
        teb.append(t_e_b_auswahlhandler(0,dialog_termin.typ))
        if klassenzahl>0: 
            teb.append(t_e_b_auswahlhandler(1,dialog_termin.typ))
        hat_ab=True
        kk_auswahl=True
    elif a_s.aS.ist_terminverwalter() and a_s.aS.ist_terminverwaltung():
        teb.append(t_e_b_auswahlhandler(2,dialog_termin.typ))
        hat_o=True
        kk_auswahl=True

    if nurQSA or (dialog_termin.tid==0 and a_s.aS.ist_kurs_verwalter()) \
              or (dialog_termin.typ==1 and kurszahl):
        teb.append(t_e_b_auswahlhandler(3,dialog_termin.typ)) #Q11-Schulaufgabe
        teb.append(t_e_b_auswahlhandler(4,dialog_termin.typ)) #Q12-Schulaufgabe
        kk_auswahl=True
    if (not nurQSA and not dialog_termin.tid) or dialog_termin.typ >=2:
        if a_s.aS.ist_terminverwalter() or a_s.aS.ist_kurs_verwalter():
            teb.append(t_e_b_auswahlhandler(5,dialog_termin.typ)) #Q11-Termin
            teb.append(t_e_b_auswahlhandler(6,dialog_termin.typ)) #Q12-Termin
            hat_c=True
            kk_auswahl=True
        if a_s.aS.ist_terminverwalter() and a_s.aS.ist_terminverwaltung():
            teb.append(t_e_b_auswahlhandler(7,dialog_termin.typ)) # Schulleitung
            teb.append(t_e_b_auswahlhandler(8,dialog_termin.typ)) # Lehrer
            teb.append(t_e_b_auswahlhandler(9,dialog_termin.typ)) # Allgemeiner Termin
            teb.append(t_e_b_auswahlhandler(10,dialog_termin.typ)) # Fremd-Termin
            hat_def=True
    teb.append('<br>')
    if dialog_termin.status==5 or a_s.aS.ist_terminverwalter():
        teb.append('<span id="Entwurf">Als Entwurf speichern <input name="entwurfsflag" type="checkbox" value="istentwurf" {} /><br></span>'.format('checked' if dialog_termin.status==5 else ''))
    if dialog_termin.typ in veroeffentlichbare_termintypen or not dialog_termin.tid:
        teb.append('<span id="jbhp">Für Jahresbericht (jb)<input name="jbflag" type="checkbox" value="fuerjb" {} /><br>'.format('checked' if dialog_termin.tag_jb else ''))
        teb.append('Für Homepage (hp)<input name="hpflag" type="checkbox" value="fuerhp" {} /><br></span>'.format('checked' if dialog_termin.tag_hp else ''))
    
    ## Rechte Seite
    teb.append('</div>\n<div  style="float: right;">\n')
    teb.append(a_tool.text_lur('Beschreibung:','' ))
    teb.append(a_tool.text_lur('','<textarea name="tkommentar" cols="35" rows="6" maxlength="'+str(TERMIN_KOMMENTARLAENGE)+'">'+dialog_termin.kommentar+'</textarea>' ))

    if (not dialog_termin.tid and not nurklassen) or dialog_termin.typ in komplexe_termintypen:
        if a_s.aS.ist_terminverwalter() or a_s.aS.ist_kurs_verwalter():
            teb.append(a_tool.text_lur('Verantwortliche:','' ))
            if hat_ab:
                teb.append(a_tool.text_lur('nicht für a) und b)','' ))
            teb.append(a_tool.text_lur('','<textarea name="tverantwortliche" cols="30" rows="3" maxlength="'+str(TERMIN_VERANTWORTLAENGE)+'">'+dialog_termin.verantwortliche+'</textarea>' ))

    if kk_auswahl:
        nbab='<br>nur bei t) und s)<br>' if (not dialog_termin.tid) and not (hat_c or hat_def)  else ""
        teb.append(a_tool.text_lur('Betroffene<br>Kurse & Klassen:'+nbab+'<br><small><i>Mehrfachauswahl<br>mit STRG und Klick!</i></small><br><span id="tkursewahl">x</span>',tkk_auswahl ))
    
    if dialog_termin.tid:
        teb.append('<input name="tid2modify" value="'+str(dialog_termin.tid)+'" type="hidden">')
    teb.append('<input name="tdatum"   value="'+dialog_termin.datum         +'" type="hidden" />') 
    teb.append('<input name="tsavesid" value="'+str(a_s.aS.sid)+'" type="hidden">')
    teb.append('<br>')
    if dialog_termin.tid:
        teb.append('<input value="Ändern" type="submit">')
    else:
        teb.append('<input value="Speichern" type="submit">')
    teb.append('</div></form></div>\n')
    a_tool.prt("".join(teb))
    
def noschooltermin2datum(datum):
    if datum.startswith('9999'):
        if int(datum[5:7])<8: # Wenn vor August
            jahr=a_s.schuljahresgrenze_iso(True)[:4]# Dann die Jahreszahl der Schuljahrendes nutzen
        else:
            jahr=a_s.schuljahresgrenze_iso(False)[:4]# Sonst die Jahreszahl des Schuljahresanfangs
        datum=jahr+datum[4:] 
    return datum
    
def export_terminkalender(code,resultat=''):
    ''' export_terminkalender liefert einen Terminkalender nach in "code" als Zeichen kodierten Kriterien, 
        welche i.d.R. direkt vom Browser angefordert werden können.'''
    codeliste={ "a": "Automatische Abfrage durch elektronisches Schulinfosystem",
                "b":  "ab Beginn des schuljahrs",
                "e":  "bis zum Ende des schuljahrs",
                "j":  "für den Jahresbericht",
                "h":  "für die Homepage",
                "l":  "für Lehrer (als zusätzliche Betroffene)",
                "n":  "n",
                "q":  "für Q-stufe (als zusätzliche Betroffene)",
                "s":  "für Schulleitung (als zusätzliche Betroffene)",
                "x":  "keine Verantwortlichen",
                "w":  "limitierte Breite (width)",
                "z":  "Keine Eingabe, minimale Formatierung" }
                    
    if "a" in code:
        a_tool.prt(a_h.einleitung()+a_h.headstart()+'''
        </head>\n<body><style type="text/css">
        .terminplaner td {    padding:5px; }
        table.terminplaner th { text-align:center;}
        tr.gerade td        { background:#FFE; }
        tr.ungerade td      { background:#EFF; }
        table, td,th {  border-style: solid;    border-width: 1px; 
                        border-color:#339;      border-spacing:0; }
        tr.kwtrenner td {   border-top:3px;
                            border-top-color:#222; border-top-style:solid;  }
        .rot, rot               {   color:red;}
        .blau, blau             {   color:blue;}
        .orange, orange         {   color:#FC6;}
        .darkorange, darkorange {   color:#A83; font-weight: bold;}
        .fremd { color:#484; font-size:90%;  }
        .entwurf { color:#C77;  }
        .redborder, .redborder td { border-right:3px solid; border-left:3px solid; border-color: red;}
        </style>
        \n''')
    elif "z" in code:
        a_tool.prt(a_h.einleitung()+'''
        <meta http-equiv="content-type" content="text/html; charset=utf-8" />
        <style type="text/css">
        body, table {
          font-family:"Helvetica Neue",helvetica,arial,sans-serif;
          font-size:12px;
        }
        table { '''+ ('max-width:715px !important;width:710px;' if "w" in code else 'max-width:80em;width:80em' ) + '''}
        table, td {
          border-style:solid;
          border-width:1px;
          border-color:#bebebe;
          border-collapse:collapse;
        }
        tr.kwtrenner td {   border-top:3px;
                            border-top-color:#222; border-top-style:solid;  }
        .redborder, .redborder td { border-right:5px solid; border-left:3px solid; border-right-color: red; border-left-color: red;}
        </style></head>\n<body>\n''')
    else:
        a_tool.prt(a_h.einleitung()+a_h.headstart()+'''
        </head>\n<body>\n''')
        
    if resultat:
        if   resultat[0]=="-":  style='background-color:#FC6;'
        elif resultat[0]=="+":  style='background-color:lightgreen;'
        elif resultat[0]=="!":  style='color:blue;'
        elif resultat[0]=="#":  style='color:darkgreen;'
        else: style=""
        a_tool.prt('<div class="s2" style="text-align:center;'+style+'">'+resultat[1:]+'</div>\n')
    mod_marker=a_tool.getcgiparameter('mod_marker')
    if len(mod_marker)==10 and a_datum.ist_isodatum(mod_marker):
        markiere_modifizierte=True
    else:
        markiere_modifizierte=False
    if "j" in code:
        merge3=True
        startdatum=a_s.schuljahresgrenze_iso()
        endedatum =a_s.schuljahresgrenze_iso(True)
    else:
        merge3=False
        startdatum=a_datum.heute_plus_delta_datum(-1)
        endedatum =a_datum.heute_plus_delta_datum(+95)

    typliste=[2,9,10] if not "-" in code else []
    code=code.lower()
    if 'q' in code: 
        typliste.append(5)
        typliste.append(6)
    if 'l' in code and not 'h' in code: typliste.append(8)
    if 's' in code and not 'h' in code: typliste.append(7)
    if 'b' in code:                     startdatum =a_s.schuljahresgrenze_iso()
    if 'e' in code:                     endedatum =a_s.schuljahresgrenze_iso(True)
    if not ( "a" in code):
        a_tool.prt('<div class="s1">')
    if not ( 'n' in code or 'j' in code):
        a_tool.prt(exportiere_naechste_schulaufgaben()+'<br>')
    keineverantwortlichen= 'x' in code
    a_tool.prt('<table class="terminplaner" ><tr>')
    a_tool.prt(A_Termin.get_kalendertitel(True,keineverantwortlichen=keineverantwortlichen,merge3=merge3)+'</tr>\n')
        
    terminliste=A_Termin.lade_liste(typliste,range(4),startdatum,endedatum,"j" in code, "h" in code)
    ferientermine_rowdict=a_db.get_rowdict_list('''SELECT erstertag,letztertag,name 
                                                    FROM noschool 
                                                    WHERE letztertag>=? AND erstertag<=?
                                                       OR letztertag>=? AND erstertag<=?
                                                ''',(startdatum,endedatum,'9999'+startdatum[4:],'9999'+endedatum[4:]))
    ferientermine=[]
    for feti in ferientermine_rowdict:
        datum=noschooltermin2datum(feti["erstertag"])
        if feti["erstertag"]==feti["letztertag"]:
            ferientermine.append([datum,'Frei am '+a_datum.wochentagXX(datum)+', '+a_datum.isodatum2datum(datum)+': <b>'+feti["name"]+'</b>'])
        else:
            ferientermine.append([datum,'<b>Ferien vom '+a_datum.wochentagXX(datum)+', '+a_datum.isodatum2datum(datum)+' bis '+a_datum.wochentagXX(feti["letztertag"])+', '+a_datum.isodatum2datum(feti["letztertag"])+': '+feti["name"]+'</b>'])
    ferientermine.sort()
    feti_zaehler=0
    oldkw=0
    merkekw=0
    letztertermin=''
    for loop_termin in terminliste:
        kw=a_datum.kalenderwoche(loop_termin.datum)
        trclass=' ungerade ' if kw%2 else ' gerade '
        if kw > oldkw:
            trclass+=' kwtrenner '
            oldkw=kw
        while feti_zaehler<len(ferientermine) and ferientermine[feti_zaehler][0]<=loop_termin.datum: ## Ferien? Ausgabe!
            a_tool.prt('<tr><td colspan=9 style="text-align:center;background:lightgreen">'+ferientermine[feti_zaehler][1]+' </td></tr>')
            feti_zaehler+=1
        if markiere_modifizierte and loop_termin.modifiziert>=mod_marker:
            trclass+= ' redborder ' 
        tr='<tr class="'+trclass+'">' 
        a_tool.prt(tr+loop_termin.als_kalenderzeile(showonly=True,keineverantwortlichen=keineverantwortlichen,merge3=merge3)+'</tr>\n')
        letztertermin=loop_termin
    a_tool.prt('</table>')
    #############################
    # und nun der Bedienbereich #
    #############################
    if not ( "a" in code or "z" in code):
        a_tool.prt('</div><div class="s2" style="float:right;">')
        a_tool.prt('<div>')
        
        gezeigt=''
        for i in sorted(typliste):
            gezeigt+='&nbsp;&nbsp;-&nbsp;'+ttypnamen[i]+'<br>'
        a_tool.prt('<h2>Terminliste</h2>Gezeigt werden Ihnen die Termine<br> vom {} bis {} für:<br>{}<br>'.format( \
            a_datum.isodatum2datum(startdatum),a_datum.isodatum2datum(endedatum),gezeigt \
            ))
        a_tool.prt('</div><div class="gradientbuttons gtermine">')
        if "h" in code:
            a_tool.prt('<div style="border:1px solid">Filter: Nur Termine, die für die Homepage getaggt sind!')
        if "j" in code:
            a_tool.prt('<div style="border:1px solid">Filter: Nur Termine, die für den Jahresbericht getaggt sind!')
        if "h" in code:
            a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code.replace('h','')+ \
        'e">Hompage-Tag-Filter <input style="width:8em;" type="submit" value=" ausschalten "></form>')
        else:
            a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code.replace('j','')+'h'+ \
        'e">Homepage-Tag-Filter <input style="width:8em;" type="submit" value=" anschalten "></form>')
        
        if "j" in code:
            a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code.replace('j','')+ \
        'e">Jahresbericht-Tag-Filter <input style="width:8em;" type="submit" value=" ausschalten "></form>')
        else:
            a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code.replace('h','')+'j'+ \
        'e">Jahresbericht-Tag-Filter <input style="width:8em;" type="submit" value=" anschalten "></form>')
        if "h" in code or "j" in code:
            a_tool.prt('</div>')
        a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code+ \
        '">Alle Änderungen ab dem<br><input type="text" name="mod_marker" size="10" value="'+ \
        a_datum.heute_plus_delta_datum(-8)+'"><input style="width:8em;" type="submit" value="markieren"></form>')
        a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code+ \
        'qls">Alle Termine (die ich sehen darf) <input style="width:8em;" type="submit" value=" anzeigen "></form>')
        if not "b" in code:
            a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code+ \
            'b">...ab Schuljahresanfang <input style="width:8em;" type="submit" value=" anzeigen "></form>')
        if not "e" in code:
            a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code+ \
            'e">...bis zum Schuljahresende <input style="width:8em;" type="submit" value=" anzeigen "></form>')
        a_tool.prt('<form action="a_termine.py"><input type="hidden" name="grab" value="'+code+ \
            'z">Für Drucker und Copy & Paste <input style="width:8em;" type="submit" value=" formatieren "></form>')
 
        a_tool.prt('</div><br>')
        a_tool.prt('''<div class="gradientbuttons"  style="float:left;">
            <a href="a_main.py">Zur Absenzenverwaltung</a>
            </div>\n''')
        a_tool.prt('''<div class="gradientbuttons gtermine"  style="float:left;">
            <a href="a_termine.py?showclassicview=1">Zum Schulaufgabenplan</a>
            </div>\n''')
        if a_s.aS.ist_terminverwalter() or a_s.aS.ist_kurs_verwalter():
            a_tool.prt('''<div class="gradientbuttons gtermine"  style="float:left;">
                <a href="a_termine.py?verwalten=1">Zur Terminverwaltung</a>
                </div>\n''')
            
    a_tool.prt('</div>')
    #a_tool.prt(', '.join( [ codeliste.get(c) for c in code ]))
    a_tool.prt('</body></html>')

    
def schulaufgabenplan(toedittid=0,showonly=False,editlevel=0):
    '''soll die Optik und Funktionalität des papiernen Schulaufgabenplans nachahmen'''
    ########## S A Plan #######    
    # Rahmen
    ########## S A Plan #######    
    ausgabe=[]
    monatslinks=''
    for i in list(range(9,13))+list(range(1,8)):
            monatslinks+='<a href="#Monat'+str(i)+'">&nbsp;&nbsp;&nbsp;'+monatsname[i][:3]+'&nbsp;&nbsp;&nbsp;</a><br>'
    ausgabe.append('<div class="gradientbuttons" style="float:right;z-index:10;position:fixed;right:0px;top:0px;text-align:center;">'+monatslinks+'</div>')
    ausgabe.append('<div class="s1 classic-view"  style="background-color:#E3E3E3;" >')
    rowdict_list=a_db.get_rowdict_list('''SELECT name,js FROM klassen ORDER BY js ASC ,name ASC''')
    klassennamen=[]
    jswechselnamen=['-']
    lastjs=-1
    for kname,js in rowdict_list:
        if js!=lastjs:
            if js in [5,7,9]:
                klassennamen.append('-')  # Platzhalter für Datum ([5,8] für Trennung U/M-Stufe)
            jswechselnamen.append(kname)
            lastjs=js
        klassennamen.append(kname)
    klassennamen.append('-')
    klassennamen.append('Q11')
    klassennamen.append('Q12')
    
    ########## S A Plan #######    
    # Tabelle, Tabellenkopf mit
    #   Schleife über alle Spalten=Klassen
    ########## S A Plan #######    
    ausgabe.append('<table class="terminplaner" style="font-size:90%">')
    zeile=['<tr>']
    for kname in klassennamen:
        if kname in jswechselnamen:
            zeile.append('<th class="linksfett">')
        else:
            zeile.append('<th>')
        zeile.append(kname+'</th>')
        titelzeile=" ".join(zeile)+'</tr>'
    ausgabe.append(titelzeile)

    ########## S A Plan #######    
    # Schleife über alle Tage mit optischer Abtrennung von Kalenderwochen und Markierung von freien Tagen
    ########## S A Plan #######    
    oldkw=-1
    oldmonat=-1
    letztes_warum=''
    warumkette=''
    erster_schultag=a_s.schuljahresgrenze_iso()
    heuteerreicht=False
    user_ist_qverwalter=a_s.aS.ist_kurs_verwalter() and not showonly
    for i in range(350):
        tag=a_datum.isodatum_plus_tage(erster_schultag,i)
        wota=a_datum.wochentag(tag)
        kw=a_datum.kalenderwoche(tag)
        monat=a_datum.kalendermonat(tag)
        
        if wota<5:
            tagesdatum=a_datum.isodatum2datum(tag)
            if tagesdatum[0]=='0': tagesdatum='&nbsp;'+tagesdatum[1:]
            lesbarertag=wotage[wota]+' '+tagesdatum[:-4]+'<small>'+tagesdatum[-4:]+'</small>'
            ob,warum=a_datum.ist_freier_tag_warum(tag)
            if ob:
                if not letztes_warum:
                    erstesdatum=lesbarertag
                if letztes_warum!=warum:
                    warumkette+=', '+warum
                letztes_warum=warum
                letztesdatum=lesbarertag
                # jetzt alles überspringen
            else:
                if letztes_warum:
                    if erstesdatum==letztesdatum:
                        zellinhalt=erstesdatum+': '+warumkette[2:]
                    else:
                        zellinhalt=erstesdatum+' - '+letztesdatum+': '+warumkette[2:]
                    ausgabe.append('<tr><td class="ferien" colspan="'+str(len(klassennamen))+'">{}</td>'.format(zellinhalt))
                    letztes_warum=''
                    warumkette=''
                if kw != oldkw:
                    if not kw%3 and oldkw>0:
                        ausgabe.append(titelzeile)
                        kwt=''
                    else:
                        kwt=' kwtrenner'
                    oldkw=kw
                else: 
                    kwt=''
                if tag==highlightdatum:
                    tr='<tr id="Highlight"'
                elif monat != oldmonat: # ggf. um einen Tag verzögert, wenn am Monatsanfang schon Highlight liegt
                    tr='<tr id="Monat'+str(monat)+'"'
                    oldmonat=monat
                elif a_datum.get_heutedatum()<=tag and not heuteerreicht: # Liegt heute auf einem Monatsanfang, so wird es als Sprungziel halt um einen Tag verschoben
                    heuteerreicht=True
                    tr='<tr id="Heute"'
                else:
                    tr='<tr'
                tr+=' class="ungerade'+kwt+'">' if kw%2 else ' class="gerade'+kwt+'">' 

                if showonly:
                    termin_obj_liste=A_Termin.lade_tages_liste((0,2,9,10),tag,auchzeitraum=True) # Liste mit Terminobjekten für diesen Tag
                else:
                    termin_obj_liste=A_Termin.lade_tages_liste((0,2,8,9,10),tag,auchzeitraum=True) # Liste mit Terminobjekten für diesen Tag
            
                terminliste=[]
                rowdict_list=a_db.get_rowdict_list('SELECT name,betroffene,tid FROM termine WHERE status>=0 AND typ=1 and datum=?',(tag,))
                for name,betroffene,tid in rowdict_list:
                    terminliste.append([name,betroffene,tid])
                zeile=[tr]
                ########## S A Plan #######    
                #   Schleife über alle Spalten=Klassen
                ########## S A Plan #######    
                for kname in klassennamen:
                    tdklasse='' 
                    tdtitel=[]
                    tdtiteltxt=''
                            
                    if kname in jswechselnamen:
                        tdklasse='linksfett '
                    zelleninhalt=[]
                    zellenhintergrund=''
                    terminflag=False
                    if kname=='-':
                        zelleninhalt=[lesbarertag]
                        tdklasse+='rechts '
                        for atermin in termin_obj_liste:
                            if atermin.status>=0:
                                tdtitel.append(atermin.als_titel())
                                terminflag=True
                    elif kname in ['Q11','Q12']:
                        rowdict_list=a_db.get_rowdict_list('SELECT betroffene,tid FROM termine WHERE status>=0 AND typ=? and datum=?',(3 if kname=='Q11' else 4,tag))
                        for betroffene,tid in rowdict_list:
                            if user_ist_qverwalter:
                                zelleninhalt.append('<a href="a_termine.py?verwalten=nein&toedittid='+str(tid)+'">'+kname+'</a>')
                            else:
                                zelleninhalt.append(kname)
                            zellenhintergrund=' style="background-color:lightgreen" '
                            tdtitel.append(a_tool.lesbare_betroffene(betroffene)+'\n')
                        if user_ist_qverwalter and not rowdict_list:
                                zelleninhalt.append('<a href="a_termine.py?verwalten=nein&tdatum='+tag+'&sa=1&zielklasse=@'+kname+'">+</a>')
                    else:
                        for name,betroffene,tid in terminliste:
                            if  kname in betroffene:
                                if showonly:
                                    zelleninhalt.append(name)
                                else:
                                    zelleninhalt.append('<a href="a_termine.py?verwalten=nein&toedittid='+str(tid)+'">'+name+'</a>')
                                zellenhintergrund=' style="background-color:lightgreen" '
                        for atermin in termin_obj_liste:
                            if (atermin.status>=0 and kname in atermin.betroffene):
                                tdtitel.append(atermin.als_titel())
                        if not showonly:
                            zelleninhalt.append('<a href="a_termine.py?verwalten=nein&tdatum='+tag+'&sa=1&zielklasse=@'+kname+'">+</a>')
                    if len(tdtitel):
                        tdtiteltxt=' title="'+'\n'.join(tdtitel)+'" '
                        if terminflag:  zellenhintergrund=' style="background-color:#FC6;" '
                    if len(tdklasse.strip()):
                        tdklasse=' class="'+tdklasse+'" '
                    zeile.append('<td'+tdklasse+tdtiteltxt+zellenhintergrund+'>'+','.join(zelleninhalt)+'</td>')
                ausgabe.append(" ".join(zeile)+'</tr>')

            

    ausgabe.append('</table>')
    return '\n'.join(ausgabe)
    
def terminkalender(toedittid=0,showonly=False,qsa_flag=False):
    '''gibt einen Kalender aus mit jeweils einer Zeile je Kalenderwoche. Rechts daneben die Termine der Woche.
    * toedittid:    ist die TerminID eines Termins, der (woanders) zum Editieren angezeigt wird. Dieser wird hier blau geschrieben
    * showonly:     verbirgt alle Editierknöpfe und gelöschten Termine
    * qsa_flag: Bei True wird ein Q-SchulAufgabenplan erstellt'''
    ausgabe=['<table class="terminplaner">']
    
    if not qsa_flag: 
        zwischenzeile='<tr><th>KW</th><th><b>Mo</b></th><th><b>Di</b></th><th><b>Mi</b></th><th><b>Do</b></th><th><b>Fr</b></th><th>Sa</th><th>So</th>'+ \
                        A_Termin.get_kalendertitel(showonly)
    else:
        zwischenzeile='<tr><th>KW</th><th width=20%><b>Mo</b></th><th><b>Di</b></th><th><b>Mi</b></th><th><b>Do</b></th><th><b>Fr</b></th><th>Sa</th><th>So</th>'
    zwischenzeile+='</tr>'
    ausgabe.append(zwischenzeile)
    heutedatum=a_datum.get_heutedatum()
    auswahldelta=28*a_s.aS.get_tbereich()
    startdelta=auswahldelta-7-a_datum.wochentag(heutedatum) # ein Montag, mindestens eine Woche vor heute
    startdatum=a_datum.heute_plus_delta_datum(startdelta) 
    wochenzaehler=0
    terminzaehler=0
    while wochenzaehler<14 and (wochenzaehler<9 or terminzaehler<40):
        if  wochenzaehler in [4,8] or \
            (terminzaehler>wochenzaehler*3 and not wochenzaehler%2) or \
            (terminzaehler>wochenzaehler*6 and wochenzaehler>0):
                ausgabe.append(zwischenzeile)
        tagesdatum=['']*7
        tagesterminanzahl=[0]*7
        tagesterminnamen=['']*7
        for delta in range(7):
            tagesdatum[delta]=a_datum.heute_plus_delta_datum(startdelta+7*wochenzaehler+delta)
            if qsa_flag:
                rowdict_list=a_db.get_rowdict_list('SELECT name,betroffene FROM termine WHERE status>=0 AND (typ=3 OR typ=4) AND datum<=? AND datumende >=?',(tagesdatum[delta],tagesdatum[delta]))
                anzahl=0
                for name,betroffene in rowdict_list:
                    anzahl+=1
                    tagesterminnamen[delta]+='\n<b>'+name+':</b><br>'+betroffene
                tagesterminanzahl[delta]=anzahl
                if not anzahl:
                    tagesterminnamen[delta]='-'
            else:
                rowdict_list=a_db.get_rowdict_list('SELECT name FROM termine WHERE status>=0 AND (typ=2 OR typ>=5) AND datum<=? AND datumende >=?',(tagesdatum[delta],tagesdatum[delta]))
                anzahl=0
                for ttname, in rowdict_list:
                    anzahl+=1
                    tagesterminnamen[delta]+='\n'+ttname
                tagesterminanzahl[delta]=anzahl
                if not anzahl:
                    tagesterminnamen[delta]='- keine Termine -'
        kw=a_datum.kalenderwoche(tagesdatum[0])
        tr='<tr class="ungerade' if kw%2 else '<tr class="gerade' 
        if not qsa_flag:
            terminzahl,termintabelle=terminliste(tagesdatum[0],tagesdatum[6],toedittid,showonly,a_s.aS.ist_terminverwalter())
            rs=(' rowspan="'+str(terminzahl)+'"') if terminzahl else ''
        else:
            terminzahl,termintabelle=0,[]
            rs=''
        ausgabe.append(tr+' kwtrenner"><td class="kalendertag" '+rs+'><i>'+str(kw)+'</i></td>')
        
        for delta in range(7):
            ob,warum=a_datum.ist_freier_tag_warum(tagesdatum[delta])
            if ob: warum='\nFrei wegen '+warum+"!"
            tdklasse='kalendertag'
            if showonly:
                tdklasse+='mini'
            if delta>4:
                tdklasse+=' atwe'
            if not qsa_flag:
                ausgabe.append('<td class="'+tdklasse+'" title="'+tagesterminnamen[delta]+warum+'"'+rs)
            else:
                ausgabe.append('<td class="'+tdklasse+'" title="'+warum+'"'+rs)
            tdstyle=''
            if tagesdatum[delta]==heutedatum:
                tdstyle='background-color:lightblue;'
            elif tagesterminanzahl[delta]:
                tdstyle='background-color:#fc6;'
            elif ob:
                tdstyle='background-color:green;'
            if delta==6:
                tdstyle+='border-right-width:3px;  border-right-style:solid;'
            if tdstyle:
                ausgabe.append(' style="'+tdstyle+'"')
            ausgabe.append('>')
            if tagesterminanzahl[delta]: ausgabe.append('<br>')
            if not showonly:
                ausgabe.append('<a href="a_termine.py?tdatum='+tagesdatum[delta]+'">')
            ausgabe.append(a_datum.isodatum2tagmonat(tagesdatum[delta]))
            if not showonly:
                ausgabe.append('</a>')
            if not qsa_flag:
                if tagesterminanzahl[delta]: ausgabe.append('<br>'+str(tagesterminanzahl[delta]))
            else:
                if tagesterminanzahl[delta]: ausgabe.append('<br>'+str(tagesterminnamen[delta]))
            ausgabe.append('</td>')
        if not qsa_flag:
            if terminzahl :
                # die erste terminzeile kommt an das Ende der Kalenderzeile
                ausgabe.append(termintabelle[0]+'</tr>') 
                if terminzahl>1:
                    for tzaehler in range(1,terminzahl):
                        # alle weiteren Terminzeilen sind eigene HTML-Rows, 
                        # da die Kalendertage via rowspan automatisch eingeblendet werden
                        ausgabe.append('\t\t\t'+tr+'">'+termintabelle[tzaehler]+'</tr>\n') 
                terminzaehler+=terminzahl
            else:
                ausgabe.append(A_Termin.get_kalenderdummy(showonly)) # bei fehlender Terminzeile: Lücke...
        else:
            ausgabe.append('</tr>')
        wochenzaehler+=1
        
    ausgabe.append('</table>\n')
    return startdatum,tagesdatum[6]," ".join(ausgabe)

    
def terminliste(modatum,sodatum,toedittid=0,showonly=False,user_ist_terminverwalter=False):
    tl=[]
    t_liste=A_Termin.lade_liste([2]+list(range(5,15)),list(range((-1 if not showonly else 0),10)),modatum,sodatum)
    zaehler=0
    for loop_termin in t_liste:
        tl.append(loop_termin.als_kalenderzeile(showonly,toedittid=0,user_ist_terminverwalter=user_ist_terminverwalter))
        zaehler+=1
    return zaehler, tl
        
def grosse_terminverwaltung(toedittid):
    gtv_dialog=[]
    gtv_dialog.append('<div class="s1 "><div class="gradientbuttons gtermine">')  #gradientbuttons
    startdatum,endedatum,tk=terminkalender(toedittid,showonly=showonly)
    vor=a_s.aS.get_tbereich()-1
    nach=a_s.aS.get_tbereich()+1
    title1='Allgemeine Termine'+(' bearbeiten &nbsp; ' if not showonly else '')
    title2='<a href="a_termine.py?verwalten='+str(vor)+'">-4 Wochen</a> &nbsp;'
    title2+=' vom '+a_datum.isodatum2datum(startdatum)+' bis '+a_datum.isodatum2datum(endedatum)+' &nbsp; &nbsp;'
    title2+='<a href="a_termine.py?verwalten='+str(nach)+'">+4 Wochen</a> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
    title2+='<a href="a_termine.py?verwalten=0">springe zu "Heute"</a> &nbsp;'
    gtv_dialog.append(a_tool.headhelper(title1,zeichenfarbe='#339',titel=title2,hilfetext=g_t_verwaltung_hilfe)+'</div>')
    gtv_dialog.append(tk+'</div>')
    return '\n'.join(gtv_dialog) 

def termine_write():
    global highlightdatum
    global return_to_classicview
    status=5 if a_tool.getcgiparameter("entwurfsflag")=="istentwurf" else 0
    tag_jb=1 if a_tool.getcgiparameter("jbflag")=="fuerjb" else 0
    tag_hp=1 if a_tool.getcgiparameter("hpflag")=="fuerhp" else 0
    tzeit=a_tool.getcgiparameter("tzeit")[:TERMIN_ZEITLAENGE]
    tkommentar=a_tool.getcgiparameter("tkommentar")[:TERMIN_KOMMENTARLAENGE]
    tname=a_tool.getcgiparameter("tname")[:TERMIN_NAMENLAENGE]
    tverantwortliche=a_tool.getcgiparameter("tverantwortliche")
    tdatum=a_tool.getcgiparameter("tdatum")
    if not a_datum.ist_isodatum(tdatum):
        tdatum=a_datum.datum2isodatum(tdatum) # arbeitet blind
        if not a_datum.ist_isodatum(tdatum):
            tdatum=''

    tdatumende=a_tool.getcgiparameter("tdatumende")
    if not a_datum.ist_isodatum(tdatumende):
        tdatumende=a_datum.datum2isodatum(tdatumende) # arbeitet blind
        if not a_datum.ist_isodatum(tdatumende):
            tdatumende=''
    typ=a_tool.getcgiinteger("ttyp",0)
    if not typ in veroeffentlichbare_termintypen: # nur bestimmte Typen eignen sich für die Veröffentlichung
        tag_jb=0
        tag_hp=0
    # Sicherheitschecks:
    if typ in [2,7,8,9] and not a_s.aS.ist_terminverwalter(): 
        return '-Fehlende Rechte für allgemeine Termine'
    if typ in [3,4,5,6] and not ( a_s.aS.ist_kurs_verwalter() or a_s.aS.ist_terminverwalter() ):
        return '-Fehlende Rechte für Q-Termine'
    # Betroffene ermitteln
    if typ in [3,4]:
        tkurse=" ".join(a_tool.getcgilist("tkurse"))
    else:
        tkurse=""
    if typ in [0,1,2]:
        tklassen="@"+" @".join(a_tool.getcgilist("tklassen"))
        if len(tklassen.strip())<2: tklassen=""
    else:
        tklassen=""
    tbetroffene=(tkurse+' '+tklassen).strip() if typ in [0,1,2,3,4] else TERMIN_BETROFFENE[typ]
    if len(tbetroffene)==0:
        if typ==0:          return '-Es muss gewählt werden, wer von dem Termin betroffen ist'
        if typ in [1,3,4]:  return '-Es muss gewählt werden, wer von der Schulaufgabe betroffen ist (und nur Oberstufenbetreuer dürfen in Q11/12 Schulaufgaben eintragen)'
    modifiziert=a_datum.get_heutedatum()
    tid2modify=a_tool.getcgiparameter('tid2modify')
    if tid2modify:
        thistory='\ngeändert von '+a_s.aS.kuerzel+' '+a_datum.isodatum2datum(a_datum.get_heutedatum())
        rowcount=a_db.execute('''UPDATE termine 
                                SET kommentar = ?,
                                    history= history || ?,
                                    typ=?,
                                    betroffene=?,
                                    verantwortliche=?,
                                    name=CASE WHEN ? != ""
                                            THEN ?
                                            ELSE name END,
                                    datumende=CASE WHEN ? != ""
                                            THEN ?
                                            ELSE datum END,
                                    zeit=?,
                                    status= ?,
                                    tag_jb=?,
                                    tag_hp=?,
                                    modifiziert=?
                                WHERE tid=?''',(tkommentar,thistory,typ,tbetroffene,tverantwortliche,tname,tname,tdatumende,tdatumende,tzeit,status,tag_jb, \
                                    tag_hp,modifiziert,tid2modify))
        if rowcount:
            tdatum=a_db.get_first_element('SELECT datum FROM termine WHERE tid=?',(tid2modify,))
            highlightdatum=tdatum
            if typ in [1,3,4]:
                return_to_classicview=True
        
            return tdatum+'+Änderung gespeichert'
        else:
            return '-Änderung nicht eintragbar'

    tautor=a_s.aS.kuerzel
    
    if tdatumende=='':
        tdatumende=tdatum
    
    if a_datum.ist_isodatum(tdatum) and tname: # Datum und Name sind Vorausetzung
        if tdatumende and (tdatumende<tdatum or tdatumende>a_datum.isodatum_plus_tage(tdatum,TERMIN_MAXINTERVALL)):
            return '-Illegales Endedatum! '+a_datum.isodatum2datum(tdatumende)+' hätte maximal '+str(TERMIN_MAXINTERVALL)+' Tage nach '+a_datum.isodatum2datum(tdatum)+' liegen müssen!'
        highlightdatum=tdatum
        if typ in [1,3,4]:
            return_to_classicview=True
        thistory='Erstellt von '+a_s.aS.kuerzel+': '+a_datum.isodatum2datum(a_datum.get_heutedatum())
        a_db.execute('''INSERT INTO termine 
                (name,kommentar,datum,datumende,zeit,typ,status,autor,betroffene,history,verantwortliche,tag_jb,tag_hp,modifiziert) 
                VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)''',(tname,tkommentar,tdatum,tdatumende,tzeit,typ,status,tautor,tbetroffene,thistory,tverantwortliche,tag_jb,tag_hp,modifiziert))
        return tdatum+'+Termin Typ '+str(typ)+' von '+tautor+' am '+tdatum+' "'+tname+'" eingetragen für '+a_tool.lesbare_betroffene(tbetroffene,True)
    else:
        fehlt=[]
        if not tdatum:
            fehlt.append('des Datums')
        if not tname:
            fehlt.append('des Termin-Namens')
        return '-Es fehlte die Angabe '+' und '.join(fehlt)
    
    
def termine_delete(tid):
    tautor='\nGelöscht von '+a_s.aS.kuerzel+' '+a_datum.isodatum2datum(a_datum.get_heutedatum())
    tdatum=a_db.get_first_element('''SELECT datum FROM termine WHERE tid=?''',(tid,))
    rowcount=a_db.execute('''UPDATE termine 
                        SET history=history || ? ,
                            status=-1 
                        WHERE tid=?''',(tautor,tid))
    if rowcount==0:
        return "-Nichts wurde gelöscht"
    else:
        return tdatum+"+Erfolgreich gelöscht" # # führt zu neuem Dialog, daher das Datum
    
def exportiere_naechste_schulaufgaben():
    tageszaehler=0
    index=0
    kopf=['<table class="terminplaner" width=100%><tr>']
    daten=['<tr>']
    trenner='<br>'
    while tageszaehler<5:
        datum=a_datum.heute_plus_delta_datum(index)
        heutemeldung= 'Heute, ' if index==0 else ''
        index+=1
        if not a_datum.ist_freier_tag(datum) and  (a_datum.wochentag(datum)<5):
            tageszaehler+=1
            termin_obj_liste=A_Termin.lade_tages_liste((1,),datum) # Liste mit Schulaugaben-Termin-Objekten für diesen Tag
            terminzaehler=0
            terminanzeige_59=''
            terminanzeige_10=''
            terminanzeige_q=''
            trenner2=''
                
            kopf.append('<td>'+a_datum.isodatum2datum(datum)[:6]+'<br><b>'+heutemeldung+wotage[a_datum.wochentag(datum)]+'</b></td>')
            for loop_termin in termin_obj_liste:
                if loop_termin.status<0:
                    continue
                terminanzeige='<b>'+a_tool.lesbare_betroffene(loop_termin.betroffene).replace(',',' ')+':</b> '+ \
                                loop_termin.name.replace('Kurzarbeit','KA')[:5]+('<small>...</small>' if len(loop_termin.name)>5 else '')+'<br>'
                if loop_termin.betroffene[:3]=='@10':
                    terminanzeige_10+=terminanzeige
                elif loop_termin.betroffene[0]=='@':
                    terminanzeige_59+=terminanzeige
                else:
                    terminanzeige_q+=terminanzeige
                terminzaehler+=1
                trenner2=trenner if ((terminanzeige_10 or terminanzeige_59) and terminanzeige_q) else ''
            daten.append('<td>'+terminanzeige_59+terminanzeige_10+trenner2+terminanzeige_q+'</td>')
    return 'Die nächsten Schulaufgabentermine:'+''.join(kopf)+'</tr>'+''.join(daten)+'</tr></table>'
            
    
if __name__ == '__main__':
    if not a_db.get_config_int('flag_termineanzeigen',0):
        a_tool.prt(a_h.einleitung()+a_h.headstart()+'</head><body><h1>Terminverwaltung ist abgeschaltet</h1></body></html>')            
        exit()
    # einmal täglich gelöschte Einträge bereinigen - mit dem ersten Aufruf
    if a_db.get_config('intern_a_termine_cleanup_date')<a_datum.get_heutedatum():
        a_db.execute('DELETE FROM termine WHERE status<0 AND typ>=5 AND modifiziert<?',(a_datum.heute_plus_delta_datum(0),))
        a_db.execute('DELETE FROM termine WHERE status<0 AND modifiziert<?',(a_datum.heute_plus_delta_datum(-7),))
        a_db.set_config('intern_a_termine_cleanup_date',a_datum.get_heutedatum())
    meldung=''
    showonly=True if a_tool.getcgiparameter('showonly')=='1'  else False
    grabbingcode=a_tool.getcgiparameter('grab')
    if grabbingcode: # es soll eine Terminliste in HTML exportiert werden
        ip=a_tool.get_client_ip()
        if ip==a_db.get_config('AdresseInfoDisplay'): # eine Infodisplay-Software darf alles lesen
            export_terminkalender('a'+grabbingcode) # das a markiert eine automatisierte Abfrage, keine Buttons gewünscht
            exit()
        else:
            erlaubt="qjhbenszw" # nur zu Testzwecken auch a um die automatische Abfrage zu testen
            a_s.check_session()
            if a_s.aS.kuerzel!='Gast':
                erlaubt+='l'
            if a_s.aS.ist_termin_schulleitung():
                erlaubt+='s'
                grabbingcode+='s'
            grabmal='' if not "j" in grabbingcode else ''
            for c in grabbingcode:
                if c in erlaubt and not (c in grabmal):
                    grabmal+=c
            export_terminkalender(grabmal)
            exit()
    a_s.check_session()
    
    scv=a_tool.getcgiparameter('showclassicview')
    if scv=='1':
        termine_dialog(showclassicview=True)
        exit()
        
    if a_s.aS.kuerzel=="Gast":
        meldung="-Kein erfolgreich angemeldeter User!"
    else:
        verwalten=a_tool.getcgiparameter("verwalten")
        #meldung=str(verwalten)
        if verwalten:
            try:
                bereich=int(verwalten)
                if a_s.aS.ist_terminverwalter() or a_s.aS.ist_kurs_verwalter() :
                    a_s.aS.setze_tbereich(bereich)
            except:
                if verwalten.lower()=="nein":
                    a_s.aS.setze_tbereich(-999)
        
        tsavesid=a_tool.getcgiparameter("tsavesid")
        tdelsid=a_tool.getcgiparameter("tdelsid")
        toedittid=a_tool.getcgiparameter("toedittid")
        kopietid=a_tool.getcgiparameter("kopietid")
        
        if tsavesid: # aha, es wurde editiert und soll nun gespeichert werden
            if tsavesid==str(a_s.aS.sid):
                meldung=termine_write()
                if a_datum.ist_isodatum(meldung[:10]):
                    termine_dialog(meldung[:10],resultat=meldung[10:],noedit=True,showclassicview=return_to_classicview)
                    exit()
            else:
                meldung="-Falsche Session-ID zum Speichern"
            termine_dialog(resultat=meldung)
            exit()

        elif tdelsid: # aha, es soll ein Termin gelöscht werden
            if tdelsid==str(a_s.aS.sid):
                tid2delete=a_tool.getcgiparameter('tid2delete')
                if tid2delete:
                    meldung=termine_delete(tid2delete)
                    if a_datum.ist_isodatum(meldung[:10]):
                        termine_dialog(meldung[:10],resultat=meldung[10:],noedit=True)
                        exit()
                else:
                    meldung="-Fehlerhafte Löschaufforderung"
            else:
                meldung="-Falsche Session-ID zum Löschen"
        elif toedittid:
            termine_dialog('',int(toedittid),resultat='!Bearbeiten Sie den Termin')
            exit()
        elif kopietid:
            termine_dialog('',int(kopietid),resultat='!Kopieren Sie den Termin',kopieanlegen=True)
            exit()
        else: # neu anlegen
            editdatum=a_tool.getcgiparameter("editdatum")
            tdatum=a_tool.getcgiparameter("tdatum")
            zielklasse=a_tool.getcgiparameter("zielklasse")
            if a_datum.ist_isodatum(editdatum):
                termine_dialog(editdatum,resultat='#Erstellen Sie einen neuen Termin für den '+a_datum.isodatum2datum(editdatum))
                exit()
            elif a_datum.ist_isodatum(tdatum):
                termine_dialog(tdatum,resultat='#Erstellen Sie einen neuen Termin für den '+a_datum.isodatum2datum(tdatum),zielklasse=zielklasse)
                exit()
            else:
                if not meldung: # hat eine vorhergehende Aktion einen Fehler gemeldet?
                    termine_dialog()
                    exit()

    export_terminkalender('x')
    exit()
    #import a_main
    #a_main.absentmain(erfolgsmeldung=meldung)   
    

